@@ -84,7 +84,8 @@ typedef struct {
 } GstPluginDep;
 
 struct _GstPluginPrivate {
-  GList *deps;    /* list of GstPluginDep structures */
+  GList *deps;                 /* list of GstPluginDep structures */
+  GstStructure *status_info;
   GstStructure *cache_data;
 };
 
@@ -113,7 +114,6 @@ G_GNUC_INTERNAL  gboolean _priv_plugin_deps_env_vars_changed (GstPlugin * plugin
 G_GNUC_INTERNAL  gboolean _priv_plugin_deps_files_changed (GstPlugin * plugin);
 
 /* init functions called from gst_init(). */
-G_GNUC_INTERNAL  void  _priv_gst_quarks_initialize (void);
 G_GNUC_INTERNAL  void  _priv_gst_mini_object_initialize (void);
 G_GNUC_INTERNAL  void  _priv_gst_memory_initialize (void);
 G_GNUC_INTERNAL  void  _priv_gst_allocator_initialize (void);
@@ -144,8 +144,12 @@ G_GNUC_INTERNAL  void  _priv_gst_caps_cleanup (void);
 G_GNUC_INTERNAL  void  _priv_gst_debug_cleanup (void);
 G_GNUC_INTERNAL  void  _priv_gst_meta_cleanup (void);
 
+G_GNUC_INTERNAL  void  _priv_gst_object_call_async (GstObject * object,
+                                                    GFunc func,
+                                                    gpointer user_data,
+                                                    GDestroyNotify notify);
 /* called from gst_task_cleanup_all(). */
-G_GNUC_INTERNAL  void  _priv_gst_element_cleanup (void);
+G_GNUC_INTERNAL  void  _priv_gst_thread_pool_cleanup (void);
 
 /* Private registry functions */
 G_GNUC_INTERNAL
@@ -154,7 +158,7 @@ gboolean _priv_gst_registry_remove_cache_plugins (GstRegistry *registry);
 G_GNUC_INTERNAL  void _priv_gst_registry_cleanup (void);
 
 GST_API
-gboolean _gst_plugin_loader_client_run (void);
+gboolean _gst_plugin_loader_client_run (const gchar * pipe_name);
 
 G_GNUC_INTERNAL  GstPlugin * _priv_gst_plugin_load_file_for_registry (const gchar *filename,
                                                                       GstRegistry * registry,
@@ -165,9 +169,9 @@ G_GNUC_INTERNAL  GstPlugin * _priv_gst_plugin_load_file_for_registry (const gcha
 G_GNUC_INTERNAL const char * _priv_gst_value_gtype_to_abbr (GType type);
 
 G_GNUC_INTERNAL gboolean _priv_gst_value_parse_string (gchar * s, gchar ** end, gchar ** next, gboolean unescape);
-G_GNUC_INTERNAL gboolean _priv_gst_value_parse_simple_string (gchar * str, gchar ** end);
+G_GNUC_INTERNAL gboolean _priv_gst_value_parse_simple_string (gchar * str, gchar ** end, char delim);
 G_GNUC_INTERNAL gboolean _priv_gst_value_parse_value (gchar * str, gchar ** after, GValue * value, GType default_type, GParamSpec *pspec);
-G_GNUC_INTERNAL gchar * _priv_gst_value_serialize_any_list (const GValue * value, const gchar * begin, const gchar * end, gboolean print_type);
+G_GNUC_INTERNAL gchar * _priv_gst_value_serialize_any_list (const GValue * value, const gchar * begin, const gchar * end, gboolean print_type, GstSerializeFlags flags);
 
 /* Used in GstBin for manual state handling */
 G_GNUC_INTERNAL  void _priv_gst_element_state_changed (GstElement *element,
@@ -182,7 +186,7 @@ gboolean  priv_gst_structure_append_to_gstring (const GstStructure * structure,
                                                 GString            * s,
                                                 GstSerializeFlags flags);
 G_GNUC_INTERNAL
-gboolean priv__gst_structure_append_template_to_gstring (GQuark field_id,
+gboolean priv__gst_structure_append_template_to_gstring (const gchar * field,
                                                         const GValue *value,
                                                         gpointer user_data);
 
@@ -243,6 +247,13 @@ GST_API gboolean _gst_disable_registry_cache;
  * as the main application in order to determine dependencies */
 GST_API gchar *_gst_executable_path;
 
+/* Internal variables used in gstutils.c and initialized in
+ * _priv_gst_plugin_initialize(). */
+G_GNUC_INTERNAL
+extern GQuark _priv_gst_plugin_api_quark;
+G_GNUC_INTERNAL
+extern GQuark _priv_gst_plugin_api_flags_quark;
+
 /* provide inline gst_g_value_get_foo_unchecked(), used in gststructure.c */
 #define DEFINE_INLINE_G_VALUE_GET_UNCHECKED(ret_type,name_type,v_field) \
 static inline ret_type                                                  \
@@ -348,12 +359,6 @@ extern GstClockTime _priv_gst_start_time;
 
 #endif
 
-#ifdef GST_DISABLE_GST_DEBUG
-/* for _gst_element_error_printf */
-#define __gst_vasprintf __gst_info_fallback_vasprintf
-int __gst_vasprintf (char **result, char const *format, va_list args);
-#endif
-
 /**** objects made opaque until the private bits have been made private ****/
 
 #include <gmodule.h>
@@ -518,7 +523,7 @@ struct _GstDynamicTypeFactoryClass {
 struct _GstClockEntryImpl
 {
   GstClockEntry entry;
-  GWeakRef clock;
+  GWeakRef *clock;
   GDestroyNotify destroy_entry;
   gpointer padding[21];                 /* padding for allowing e.g. systemclock
                                          * to add data in lieu of overridable
@@ -526,6 +531,14 @@ struct _GstClockEntryImpl
 };
 
 char * priv_gst_get_relocated_libgstreamer (void);
+gint   priv_gst_count_directories (const char *filepath);
+
+void priv_gst_clock_init (void);
+GstClockTime priv_gst_get_monotonic_time (void);
+GstClockTime priv_gst_get_real_time (void);
+
+/* Needed for hashing in gstvalue.c */
+G_GNUC_INTERNAL const GstStructure * _gst_tag_list_structure (const GstTagList * list);
 
 G_END_DECLS
 #endif /* __GST_PRIVATE_H__ */
