@@ -48,18 +48,17 @@ typedef struct _GstTracerFactoryClass GstTracerFactoryClass;
 
 /* tracering interface */
 
-GST_EXPORT
+GST_API
 GType           gst_tracer_factory_get_type          (void);
 
-GST_EXPORT
+GST_API
 GList *         gst_tracer_factory_get_list          (void);
 
-GST_EXPORT
+GST_API
 GType           gst_tracer_factory_get_tracer_type   (GstTracerFactory * factory);
 
-#ifdef G_DEFINE_AUTOPTR_CLEANUP_FUNC
+
 G_DEFINE_AUTOPTR_CLEANUP_FUNC(GstTracerFactory, gst_object_unref)
-#endif
 
 G_END_DECLS
 
