@@ -448,9 +448,10 @@ gst_type_find_helper_get_range_full (GstObject * obj, GstObject * parent,
 
   *caps = result;
   if (helper.flow_ret == GST_FLOW_EOS) {
-    /* Some typefinder might've tried to read too much, if we
-     * didn't get any meaningful caps because of that this is
-     * just a normal error */
+    /* Some typefinder might've tried to read too much, especially if
+     * dealing with small sources or with limited num-buffers. In
+     * this case, EOS is received, and it is converted to ERROR
+     * because the downstream peer is not yet connected to send it. */
     helper.flow_ret = GST_FLOW_ERROR;
   }
 
@@ -579,6 +580,14 @@ buf_helper_find_suggest (gpointer data, guint probability, GstCaps * caps)
   }
 }
 
+static guint64
+buf_helper_get_length (gpointer data)
+{
+  GstTypeFindBufHelper *helper = (GstTypeFindBufHelper *) data;
+
+  return helper->size;
+}
+
 /**
  * gst_type_find_helper_for_data:
  * @obj: (nullable): object doing the typefinding, or %NULL (used for logging)
@@ -670,7 +679,7 @@ gst_type_find_helper_for_data_with_extension (GstObject * obj,
   find.data = &helper;
   find.peek = buf_helper_find_peek;
   find.suggest = buf_helper_find_suggest;
-  find.get_length = NULL;
+  find.get_length = buf_helper_get_length;
 
   type_list = gst_type_find_factory_get_list ();
   type_list = prioritize_extension (obj, type_list, extension);
