@@ -8,6 +8,8 @@
 //
 // SPDX-License-Identifier: MPL-2.0
 
+#![allow(unused)]
+
 use std::io;
 
 pub trait ReadBytesBEExt: io::Read {
@@ -187,10 +189,10 @@ impl PtpMessage {
 
         let message_payload = match message_type {
             PtpMessageType::ANNOUNCE => {
-                if data.len() < 20 {
+                if data.len() < 30 {
                     return Err(io::Error::new(
                         io::ErrorKind::InvalidInput,
-                        format!("Too short PTP SYNC message {} < 20", data.len(),),
+                        format!("Too short PTP ANNOUNCE message {} < 20", data.len(),),
                     ));
                 }
 
@@ -246,7 +248,7 @@ impl PtpMessage {
                 if data.len() < 10 {
                     return Err(io::Error::new(
                         io::ErrorKind::InvalidInput,
-                        format!("Too short PTP SYNC message {} < 20", data.len(),),
+                        format!("Too short PTP FOLLOW_UP message {} < 20", data.len(),),
                     ));
                 }
 
@@ -264,7 +266,7 @@ impl PtpMessage {
                 if data.len() < 10 {
                     return Err(io::Error::new(
                         io::ErrorKind::InvalidInput,
-                        format!("Too short PTP SYNC message {} < 20", data.len(),),
+                        format!("Too short PTP DELAY_REQ message {} < 20", data.len(),),
                     ));
                 }
 
@@ -280,7 +282,7 @@ impl PtpMessage {
                 if data.len() < 20 {
                     return Err(io::Error::new(
                         io::ErrorKind::InvalidInput,
-                        format!("Too short PTP SYNC message {} < 20", data.len(),),
+                        format!("Too short PTP DELAY_RESP message {} < 20", data.len(),),
                     ));
                 }
 
