@@ -57,6 +57,28 @@
 
 G_STATIC_ASSERT ((1 << BUFFER_FLAG_SHIFT) == GST_MINI_OBJECT_FLAG_LAST);
 
+#ifndef GST_DISABLE_GST_DEBUG
+#define GST_CAT_DEFAULT ensure_debug_category()
+static GstDebugCategory *
+ensure_debug_category (void)
+{
+  static gsize cat_gonce = 0;
+
+  if (g_once_init_enter (&cat_gonce)) {
+    gsize cat_done;
+
+    cat_done = (gsize) _gst_debug_category_new ("elements-private", 0,
+        "elements-private");
+
+    g_once_init_leave (&cat_gonce, cat_done);
+  }
+
+  return (GstDebugCategory *) cat_gonce;
+}
+#else
+#define ensure_debug_category() /* NOOP */
+#endif /* GST_DISABLE_GST_DEBUG */
+
 /* Returns a newly allocated string describing the flags on this buffer */
 gchar *
 gst_buffer_get_flags_string (GstBuffer * buffer)
@@ -195,6 +217,10 @@ gst_writev (gint fd, const struct iovec *iov, gint iovcnt, gsize total_bytes)
         do {
           ret = write (fd, iov[i].iov_base, iov[i].iov_len);
         } while (ret < 0 && errno == EINTR);
+
+        if (ret < 0)
+          return ret;
+
         if (ret > 0)
           written += ret;
         if (ret != iov[i].iov_len)
@@ -234,7 +260,7 @@ gst_writev_iovecs (GstObject * sink, gint fd, GstPoll * fdset,
         flow_ret = GST_FLOW_FLUSHING;
         goto out;
       }
-#ifndef HAVE_WIN32
+#ifndef G_OS_WIN32
       if (fdset != NULL) {
         do {
           GST_DEBUG_OBJECT (sink, "going into select, have %" G_GSSIZE_FORMAT
@@ -298,7 +324,7 @@ gst_writev_iovecs (GstObject * sink, gint fd, GstPoll * fdset,
       } else {
         goto write_error;
       }
-#ifdef HAVE_WIN32
+#ifdef G_OS_WIN32
       /* do short sleep on windows where we don't use gst_poll(),
        * to avoid excessive busy looping */
       if (fdset != NULL)
@@ -315,7 +341,7 @@ out:
   return flow_ret;
 
 /* ERRORS */
-#ifndef HAVE_WIN32
+#ifndef G_OS_WIN32
 select_error:
   {
     GST_ELEMENT_ERROR (sink, RESOURCE, READ, (NULL),
